"use strict";
// This tests the basic consumer and producer behaviour
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const consumer_1 = require("../backends/mongoRedis/consumer");
const producer_1 = require("../backends/mongoRedis/producer");
const mongoUrl = "mongodb://localhost:27017";
const databaseName = "topics-test";
const redisUrl = "redis://localhost:6379";
const randomId = Math.random();
const collectionName = "example-" + randomId;
const tc = new consumer_1.MongoRedisTopicConsumer(mongoUrl, databaseName, collectionName, redisUrl);
const tp = new producer_1.MongoRedisTopicProducer(mongoUrl, databaseName, collectionName, redisUrl);
const waitSecs = (secs) => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => {
        setTimeout(() => {
            resolve(null);
        }, secs * 1000);
    });
});
beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
    jest.setTimeout(30 * 1000);
    yield tc.start();
    yield tp.start();
}));
afterAll(() => __awaiter(void 0, void 0, void 0, function* () {
    yield tc.stop();
    yield tp.stop();
}));
describe("Basic operations", () => {
    let foundMessages = 0;
    test("Should consume nothing at start", () => __awaiter(void 0, void 0, void 0, function* () {
        // Consumer
        yield new Promise((resolve, reject) => {
            tc.streamMessagesFrom((msg) => {
                console.log("MSG: " + JSON.stringify(msg));
                foundMessages++;
            }, null, {}, () => {
                // All messages done
                resolve(null);
            });
        });
        yield waitSecs(5);
        expect(foundMessages).toEqual(0);
        // Producer
        for (let i = 1; i <= 10; i++) {
            yield tp.pushMessageToTopic({
                phil: randomId + " message " + i,
                isGreat: i > 5,
            });
            console.log("Successfully pushed message: " + i);
        }
        yield waitSecs(5);
        expect(foundMessages).toEqual(10);
    }));
});
