"use strict";
// To test the sole setter, we write the same data and make sure it is ignored properly
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const consumer_1 = require("../backends/mongoRedis/consumer");
const producer_1 = require("../backends/mongoRedis/producer");
const soleTopicSetter_1 = __importDefault(require("../soleTopicSetter"));
const mongoUrl = "mongodb://localhost:27017";
const databaseName = "topics-test";
const redisUrl = "redis://localhost:6379";
const randomId = Math.random();
const collectionName = "example-" + randomId;
const tc = new consumer_1.MongoRedisTopicConsumer(mongoUrl, databaseName, collectionName, redisUrl);
const tp = new producer_1.MongoRedisTopicProducer(mongoUrl, databaseName, collectionName, redisUrl);
const sts = new soleTopicSetter_1.default(tc, tp);
const tc2 = new consumer_1.MongoRedisTopicConsumer(mongoUrl, databaseName, collectionName, redisUrl);
const waitSecs = (secs) => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => {
        setTimeout(() => {
            resolve(null);
        }, secs * 1000);
    });
});
beforeAll(() => __awaiter(void 0, void 0, void 0, function* () {
    jest.setTimeout(30 * 1000);
    yield sts.start();
    yield tc2.start();
}));
afterAll(() => __awaiter(void 0, void 0, void 0, function* () {
    yield sts.stop();
    yield tc2.stop();
}));
const secondsToWaitForUpdate = 2;
describe("Setting", () => {
    let foundMessages = 0;
    test("Everything", () => __awaiter(void 0, void 0, void 0, function* () {
        // Consumer
        yield new Promise((resolve, reject) => {
            tc2.streamMessagesFrom((msg) => {
                console.log("MSG: " + JSON.stringify(msg));
                foundMessages++;
            }, null, {}, () => {
                // All messages done
                console.log("All messages done!!!...");
                resolve(null);
            });
        });
        sts.setPayload({
            phil: "phil",
            isGreat: true,
        });
        yield waitSecs(secondsToWaitForUpdate);
        expect(foundMessages).toEqual(1);
        sts.setPayload({
            phil: "phil",
            isGreat: true,
        });
        yield waitSecs(secondsToWaitForUpdate);
        expect(foundMessages).toEqual(2);
        sts.setLogCompactedPayload("phil", {
            phil: "phil",
            isGreat: true,
        });
        yield waitSecs(secondsToWaitForUpdate);
        expect(foundMessages).toEqual(3);
        sts.setLogCompactedPayload("phil", {
            phil: "phil",
            isGreat: true,
        });
        yield waitSecs(secondsToWaitForUpdate);
        expect(foundMessages).toEqual(3);
        sts.setLogCompactedPayload("phil", {
            phil: "phil",
            isGreat: false,
        });
        yield waitSecs(secondsToWaitForUpdate);
        expect(foundMessages).toEqual(4);
    }));
});
