// This tests the basic consumer and producer behaviour

import { MongoRedisTopicConsumer } from "../backends/mongoRedis/consumer";
import { MongoRedisTopicProducer } from "../backends/mongoRedis/producer";

const mongoUrl = "mongodb://localhost:27017";
const databaseName = "topics-test";
const redisUrl = "redis://localhost:6379";

type ExampleMessage = {
    phil: string
    isGreat: boolean
}

const randomId = Math.random();
const collectionName = "example-" + randomId;
const tc = new MongoRedisTopicConsumer<ExampleMessage>(mongoUrl, databaseName, collectionName, redisUrl);
const tp = new MongoRedisTopicProducer<ExampleMessage>(mongoUrl, databaseName, collectionName, redisUrl);

const waitSecs = async (secs: number) => {
    return new Promise((resolve, reject) => {
        setTimeout(() => {
            resolve(null);
        }, secs * 1000);
    })
}

beforeAll(async () => {
    jest.setTimeout(30 * 1000);
    await tc.start();
    await tp.start();    
});

afterAll(async () => {
    await tc.stop();
    await tp.stop();
});

describe("Basic operations", () => {
    let foundMessages = 0;
    test("Should consume nothing at start", async () => {

        // Consumer
        await new Promise((resolve, reject) => {
            tc.streamMessagesFrom((msg) => {
                console.log("MSG: " + JSON.stringify(msg));
                foundMessages++;
            }, null, {}, () => {
                // All messages done
                resolve(null);  
            });
        });

        await waitSecs(5);
        expect(foundMessages).toEqual(0);
    
        // Producer
        for (let i=1; i<=10; i++) {
            await tp.pushMessageToTopic({
                phil: randomId + " message " + i,
                isGreat: i > 5,
            });
            console.log("Successfully pushed message: " + i);
        }

        await waitSecs(5);
        expect(foundMessages).toEqual(10);
    })
});
