// To test the sole setter, we write the same data and make sure it is ignored properly

import { MongoRedisTopicConsumer } from "../backends/mongoRedis/consumer";
import { MongoRedisTopicProducer } from "../backends/mongoRedis/producer";
import SoleTopicSetter from "../soleTopicSetter";

const mongoUrl = "mongodb://localhost:27017";
const databaseName = "topics-test";
const redisUrl = "redis://localhost:6379";

type ExampleMessage = {
    phil: string
    isGreat: boolean
}

const randomId = Math.random();
const collectionName = "example-" + randomId;
const tc = new MongoRedisTopicConsumer<ExampleMessage>(mongoUrl, databaseName, collectionName, redisUrl);
const tp = new MongoRedisTopicProducer<ExampleMessage>(mongoUrl, databaseName, collectionName, redisUrl);
const sts = new SoleTopicSetter(tc, tp);

const tc2 = new MongoRedisTopicConsumer<ExampleMessage>(mongoUrl, databaseName, collectionName, redisUrl);

const waitSecs = async (secs: number) => {
    return new Promise((resolve, reject) => {
        setTimeout(() => {
            resolve(null);
        }, secs * 1000);
    })
}

beforeAll(async () => {
    jest.setTimeout(30 * 1000);
    await sts.start();
    await tc2.start();
});

afterAll(async () => {
    await sts.stop();
    await tc2.stop();
});

const secondsToWaitForUpdate = 2;

describe("Setting", () => {
    let foundMessages = 0;
    test("Everything", async () => {

        // Consumer
        await new Promise((resolve, reject) => {
            tc2.streamMessagesFrom((msg) => {
                console.log("MSG: " + JSON.stringify(msg));
                foundMessages++;
            }, null, {}, () => {
                // All messages done
                console.log("All messages done!!!...")
                resolve(null);  
            });
        });

        sts.setPayload({
            phil: "phil",
            isGreat: true,
        });

        await waitSecs(secondsToWaitForUpdate);
        expect(foundMessages).toEqual(1);

        sts.setPayload({
            phil: "phil",
            isGreat: true,
        });

        await waitSecs(secondsToWaitForUpdate);
        expect(foundMessages).toEqual(2);

        sts.setLogCompactedPayload("phil", {
            phil: "phil",
            isGreat: true,
        });
    
        await waitSecs(secondsToWaitForUpdate);
        expect(foundMessages).toEqual(3);

        sts.setLogCompactedPayload("phil", {
            phil: "phil",
            isGreat: true,
        });

        await waitSecs(secondsToWaitForUpdate);
        expect(foundMessages).toEqual(3);

        sts.setLogCompactedPayload("phil", {
            phil: "phil",
            isGreat: false,
        });

        await waitSecs(secondsToWaitForUpdate);
        expect(foundMessages).toEqual(4);
    });
});
